@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -8,6 +8,7 @@
 
 
 
+
 fill <-
 fill1 <- fill2 <- fill3 <-
   function(x, values = 0, ncolx = ncol(x)) {
@@ -240,7 +241,7 @@ veigen <- function(x, M) {
   index <- iam(NA, NA, M = M, both = TRUE, diag = TRUE)
   dimm.value <- nrow(x)  # usually M or M(M+1)/2
 
-  z <- .Fortran("veigen",
+  z <- .Fortran("veigenf",
       as.integer(M),
       as.integer(n),
       as.double(x),
