@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -25,7 +25,7 @@ vnonlinear.control <- function(save.weights = TRUE, ...) {
 subset.lohi <- function(xvec, yvec,
                         probs.x = c(0.15, 0.85),
                         type = c("median", "wtmean", "unwtmean"),
-                        wtvec = rep(1, len = length(xvec))) {
+                        wtvec = rep_len(1, length(xvec))) {
 
 
   if (!is.Numeric(probs.x, length.arg = 2))
@@ -173,7 +173,7 @@ micmen.control <- function(save.weights = TRUE, ...) {
 
     if (!length(Xm2))
       stop("regressor not found")
-    if (ncol(as.matrix(Xm2)) != 1)
+    if (NCOL(Xm2) != 1)
       stop("regressor not found or is not a vector. Use the ",
            "'form2' argument without an intercept")
     Xm2 <- as.vector(Xm2)  # Make sure
@@ -212,8 +212,8 @@ micmen.control <- function(save.weights = TRUE, ...) {
       if (length( .init2 )) init2 <- .init2
 
       etastart <- cbind(
-          rep(theta2eta(init1, .link1 , earg = .earg1 ), len = n),
-          rep(theta2eta(init2, .link2 , earg = .earg2 ), len = n))
+          rep_len(theta2eta(init1, .link1 , earg = .earg1 ), n),
+          rep_len(theta2eta(init2, .link2 , earg = .earg2 ), n))
     } else {
       stop("cannot handle 'etastart' or 'mustart'")
     }
@@ -238,7 +238,8 @@ micmen.control <- function(save.weights = TRUE, ...) {
     fit$df.residual <- n - rank   # Not nrow.X.vlm - rank
     fit$df.total <- n             # Not nrow.X.vlm
 
-    extra$Xm2 <- NULL             # Regressor is in control$regressor 
+
+
     dpar <- .dispersion
     if (!dpar) {
       dpar <- sum(c(w) * (y - mu)^2) / (n - ncol.X.vlm)
@@ -267,6 +268,17 @@ micmen.control <- function(save.weights = TRUE, ...) {
   summary.dispersion = FALSE,
 
   vfamily = c("micmen", "vnonlinear"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    theta1 <- eta2theta(eta[, 1], .link1 , earg = .earg1 )
+    theta2 <- eta2theta(eta[, 2], .link2 , earg = .earg2 )
+    okay1 <- all(is.finite(theta1)) &&
+             all(is.finite(theta2))
+    okay1
+  }, list( .link1 = link1, .earg1 = earg1,
+           .link2 = link2, .earg2 = earg2,
+           .firstDeriv = firstDeriv,
+           .rpar = rpar, .divisor = divisor ))),
+
 
   deriv = eval(substitute(expression({
     theta1 <- eta2theta(eta[, 1], .link1 , earg = .earg1 )
@@ -302,13 +314,19 @@ micmen.control <- function(save.weights = TRUE, ...) {
           temp200809[, 2:M] <- temp200809[, 2:M] + sqrt(rpar)
         c(w) * (y - mu) * temp200809
       } else {
-        c(w) * (y - mu) *
+        c(w) * c(y - mu) *
           cbind(dmus.dthetas[, 1] * dthetas.detas[, 1],
                 dmus.dthetas[, 2] * dthetas.detas[, 2] + sqrt(rpar))
       }
     } else {
       temp20101111 <- dmus.dthetas * dthetas.detas
-      c(w) * (y - mu) * temp20101111
+ print("head(temp20101111) in @deriv")
+ print( head(temp20101111) )
+ print("head(y) in @deriv")
+ print( head(y) )
+ print("head(mu) in @deriv")
+ print( head(mu) )
+      c(w) * c(y - mu) * temp20101111
     }
 
     myderiv
@@ -336,7 +354,7 @@ micmen.control <- function(save.weights = TRUE, ...) {
       } else {
         wz <- cbind(( dmus.dthetas[, 1] * dthetas.detas[, 1])^2,
                     ( dmus.dthetas[, 2] * dthetas.detas[, 2])^2 + rpar,
-                      dmus.dthetas[, 1] *  dmus.dthetas[, 2] * 
+                      dmus.dthetas[, 1] *  dmus.dthetas[, 2] *
                      dthetas.detas[, 1] * dthetas.detas[, 2])
       }
     } else {
@@ -364,7 +382,7 @@ micmen.control <- function(save.weights = TRUE, ...) {
   }), list( .link1 = link1, .link2 = link2,
             .firstDeriv = firstDeriv,
             .nsimEIM = nsimEIM, .oim = oim ))))
-}
+}  # micmen
 
 
 
@@ -469,7 +487,7 @@ skira.control <- function(save.weights = TRUE, ...) {
 
 
     if (!length(Xm2)) stop("regressor not found")
-    if (ncol(as.matrix(Xm2)) != 1)
+    if (NCOL(Xm2) != 1)
       stop("regressor not found or is not a vector. ",
          "Use the 'form2' argument without an intercept")
     Xm2 <- as.vector(Xm2)
@@ -545,15 +563,15 @@ skira.control <- function(save.weights = TRUE, ...) {
  points(Xm2, 1 / (init1 + init2 * Xm2), col = "green")
  } else {
  with(mysubset,
- points(c(x1bar, x2bar), c(y1bar, y2bar), col = "red", pch = "+", cex = 2))
+ points(c(x1bar, x2bar), c(y1bar, y2bar), col = 2, pch = "+", cex = 2))
  }
 
 
       if (length( .init1 )) init1 <- .init1
       if (length( .init2 )) init2 <- .init2
       etastart <- cbind(
-          rep(theta2eta(init1, .link1 , earg = .earg1 ), len = n),
-          rep(theta2eta(init2, .link2 , earg = .earg2 ), len = n))
+          rep_len(theta2eta(init1, .link1 , earg = .earg1 ), n),
+          rep_len(theta2eta(init2, .link2 , earg = .earg2 ), n))
     } else {
       stop("cannot handle 'etastart' or 'mustart'")
     }
@@ -598,6 +616,16 @@ skira.control <- function(save.weights = TRUE, ...) {
             .estimated.dispersion = estimated.dispersion ))),
   summary.dispersion = FALSE,
   vfamily = c("skira", "vnonlinear"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    theta1 <- eta2theta(eta[, 1], .link1 , earg = .earg1 )
+    theta2 <- eta2theta(eta[, 2], .link2 , earg = .earg2 )
+    okay1 <- all(is.finite(theta1)) &&
+             all(is.finite(theta2))
+    okay1
+  }, list( .link1 = link1, .earg1 = earg1,
+           .link2 = link2, .earg2 = earg2,
+           .firstDeriv = firstDeriv,
+           .rpar = rpar, .divisor = divisor ))),
   deriv = eval(substitute(expression({
     rpar <- if ( .firstDeriv == "rpar") {
       if (iter > 1) {
@@ -664,7 +692,7 @@ skira.control <- function(save.weights = TRUE, ...) {
         ysim <- 1 / (theta1 + theta2 * Xm2) + rnorm(n, sd = mysigma)
         temp3 <- (ysim - mu) * dmus.dthetas * dthetas.detas
         run.varcov <- run.varcov +
-                      temp3[, index0$row.index] * temp3[, index0$col.index]
+                  temp3[, index0$row.index] * temp3[, index0$col.index]
       }
       run.varcov <- run.varcov / .nsimEIM
 
@@ -678,7 +706,7 @@ skira.control <- function(save.weights = TRUE, ...) {
   }), list( .link1 = link1, .link2 = link2,
             .firstDeriv = firstDeriv,
             .nsimEIM = nsimEIM, .oim = oim ))))
-}
+}  # skira
 
 
 
