@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -11,19 +11,21 @@
 
 
 
-vgam <- function(formula,
-                 family, data = list(),
-                 weights = NULL, subset = NULL, na.action = na.fail,
-                 etastart = NULL, mustart = NULL, coefstart = NULL,
-                 control = vgam.control(...),
-                 offset = NULL,
-                 method = "vgam.fit",
-                 model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                 contrasts = NULL,
-                 constraints = NULL,
-                 extra = list(),
-                 form2 = NULL,  # Added 20130730
-                 qr.arg = FALSE, smart = TRUE, ...) {
+vgam <-
+  function(formula,
+           family = stop("argument 'family' needs to be assigned"),
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           etastart = NULL, mustart = NULL, coefstart = NULL,
+           control = vgam.control(...),
+           offset = NULL,
+           method = "vgam.fit",
+           model = FALSE, x.arg = TRUE, y.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = list(),
+           form2 = NULL,  # Added 20130730
+           qr.arg = FALSE, smart = TRUE, ...) {
   dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "vgam"
 
@@ -84,11 +86,11 @@ vgam <- function(formula,
     Xm2 <- retlist$Xm2
 
     if (length(Ym2)) {
-      if (nrow(as.matrix(Ym2)) != nrow(as.matrix(y)))
+      if (NROW(Ym2) != NROW(y))
         stop("number of rows of 'y' and 'Ym2' are unequal")
     }
     if (length(Xm2)) {
-      if (nrow(as.matrix(Xm2)) != nrow(as.matrix(x)))
+      if (NROW(Xm2) != NROW(x))
         stop("number of rows of 'x' and 'Xm2' are unequal")
     }
   } else {
@@ -130,7 +132,7 @@ vgam <- function(formula,
   w <- model.weights(mf)
   if (!length(w)) {
     w <- rep_len(1, nrow(mf))
-  } else if (ncol(as.matrix(w)) == 1 && any(w < 0))
+  } else if (NCOL(w) == 1 && any(w < 0))
     stop("negative weights not allowed")
 
 
